from amino import Nothing

from ribosome.machine.message_base import message

Nop = message('Nop')
Stop = message('Stop')
Quit = message('Quit')
Done = message('Done')
PlugCommand = message('PlugCommand', 'plug', 'msg')
NvimIOTask = message('NvimIOTask', 'io')
RunTask = message('RunTask', 'task', opt_fields=(('msg', Nothing),))
RunIO = RunTask
RunIOAlg = message('RunIOAlg', 'io', opt_fields=(('msg', Nothing),))
UnitTask = message('UnitTask', 'task', opt_fields=(('msg', Nothing),))
DataTask = message('DataTask', 'cons', opt_fields=(('msg', Nothing),))
ShowLogInfo = message('ShowLogInfo')
RunIOsParallel = message('RunIOsParallel', 'ios')
RunCorosParallel = message('RunCorosParallel', 'coros')
SubProcessSync = message('SubProcessSync', 'job', 'result')
Error = message('Error', 'message')
Warning = message('Warning', 'message')
Debug = message('Debug', 'message')
Coroutine = message('Coroutine', 'coro')
TransitionException = message('TransitionException', 'context', 'exc')

__all__ = ('Nop', 'Stop', 'Quit', 'Done', 'Done', 'PlugCommand', 'NvimIOTask', 'RunTask', 'UnitTask', 'DataTask',
           'ShowLogInfo', 'RunIOsParallel', 'RunCorosParallel', 'SubProcessSync', 'RunIO', 'RunIOAlg', 'Error',
           'Warning', 'Debug', 'Coroutine', 'TransitionException')



from kallikrein import Expectation
from kallikrein.matchers import contain

from amino import Either, Right

from integration._support.handler import HandlerSpecPlugin
from integration._support.base import IntegrationSpecBase


class HandlerSpec(IntegrationSpecBase):
    '''typed transition handler
    test $test
    '''

    @property
    def plugin_class(self) -> Either[str, type]:
        return Right(HandlerSpecPlugin)

    def test(self) -> Expectation:
        self.vim.cmd('HandlerStart')
        self.vim.cmd_sync('Msg')
        return self._log_line(-1, contain('nothing'))

__all__ = ('HandlerSpec',)



from ribosome import NvimStatePlugin

from amino.lazy import lazy
from amino import List, Right, Left, Either, Id, IO
from amino.state import IdState, StateT
from ribosome.logging import Logging
from ribosome.request import msg_command
from ribosome.machine import message
from ribosome.machine.state import UnloopedRootMachine, SubMachine, SubTransitions
from ribosome.nvim import NvimFacade, HasNvim, AsyncVimProxy
from ribosome.data import Data
from ribosome.record import field
from ribosome.machine.base import MachineBase
from ribosome.machine import trans


Msg = message('Msg')
Msg2 = message('Msg2')


class Env(Data):
    vim = field(AsyncVimProxy)


class HTrans(SubTransitions, HasNvim, Logging):

    def __init__(self, machine, *a, **kw) -> None:
        SubTransitions.__init__(self, machine, *a, **kw)
        HasNvim.__init__(self, machine.vim)

    @trans.one(Msg, trans.e, trans.st, trans.io)
    def msg(self) -> Either[str, StateT[Id, Env, IO[Msg2]]]:
        return Right(IdState.pure(IO.pure(Msg2())))

    @trans.one(Msg2, trans.e)
    def msg2(self) -> Either[str, Msg]:
        return Left('nothing')


class Plugin(SubMachine, HasNvim, Logging):
    Transitions = HTrans

    def __init__(self, vim: NvimFacade, parent=None, title=None) -> None:
        MachineBase.__init__(self, parent, title=title)
        HasNvim.__init__(self, vim)


class Mach(UnloopedRootMachine):
    _data_type = Env

    def __init__(self, vim: NvimFacade, **kw) -> None:
        plug = 'integration._support.handler'
        UnloopedRootMachine.__init__(self, vim, List(plug), **kw)

    @property
    def title(self):
        return 'mach'

    def init(self):
        return Env(vim=self.vim)


class HandlerSpecPlugin(NvimStatePlugin, Logging, name='handler'):

    @lazy
    def state(self) -> Mach:
        return Mach(self.vim.proxy, title='handler')

    def start_plugin(self) -> None:
        self.state.start()

    @msg_command(Msg, sync=True)
    def msg(self):
        pass

__all__ = ('HandlerSpecPlugin',)
